#include "globals.h"
#include "Mesh.h"
#include "plyio.h"

Mesh::Mesh()
{
    numVerts = 0;
    verts = NULL;
    
    numTris = 0;
    tris = NULL;

    hasConfidence = 0;
    hasIntensity = 0;
    hasNormals = 0;
    hasColor = 0;
}


Mesh::~Mesh()
{
    if (verts != NULL) {
	for (int i = 0; i < numVerts; i++) {
	    delete [] verts[i].verts;
	    delete [] verts[i].edgeLengths;
	    delete [] verts[i].tris;
	}

	delete [] verts;
    }

    if (tris != NULL) {
	delete [] tris;
    }
}



int
Mesh::readPly(char *filename)
{
   return readMeshPly(this, filename);
}


int
Mesh::writePly(char *filename)
{
   return writeMeshPly(this, filename);
}


// Compute axis-aligned bounding box for whole mesh
void
Mesh::computeBBox()
{
    bbox.init();

    Vertex *buf = this->verts;
    for (int i = 0; i < this->numVerts; i++, buf++) {
	bbox.update(buf->coord);
    }
}


// Compute triangle and vertex normals.
void
Mesh::initNormals()
{
    computeTriNormals();
    computeVertNormals();
}


// Compute triangle normals.
void
Mesh::computeTriNormals()
{
    Vec3f v1, v2, v3;
    Triangle *tri;
    int i;

    for (i = 0; i < numTris; i++) {
	tri = &tris[i];
	v1.setValue(verts[tri->vindex1].coord);
	v2.setValue(verts[tri->vindex2].coord);
	v3.setValue(verts[tri->vindex3].coord);
	v2 = v1 - v2;
	v3 = v1 - v3;
	tri->norm = v2.cross(v3);
	tri->norm.normalize();
    }
}


// Compute vertex normals as average of triangle normals.
void
Mesh::computeVertNormals()
{
    int i, index;

    for (i = 0; i < numVerts; i++) {
	verts[i].norm.setValue(0, 0, 0);
    }

    for (i = 0; i < numTris; i++) {
	index = tris[i].vindex1;
	verts[index].norm += tris[i].norm;
	
	index = tris[i].vindex2;
	verts[index].norm += tris[i].norm;
	
	index = tris[i].vindex3;
	verts[index].norm += tris[i].norm;
    }

    for (i = 0; i < numVerts; i++) {
	verts[i].norm.normalize();
    }
}


// Apply a transformation to all the vertices and normals of a mesh
//
// Note: should use inverse transpose of upper 3x3 for normals.  Just
// using upper 3x3 for now.
//
void
Mesh::applyXform(Matrix4f &mat)
{
   Vec3f vec, dir;

   Vertex *vert = verts;
   for (int i = 0; i < numVerts; i++) {
      mat.multVec(vert->coord, vec);
      vert->coord = vec;

      if (hasNormals) {
	 mat.multDir(vert->norm, dir);
	 vert->norm = dir;
	 vert->norm.normalize();
      }
      vert++;
   }
}


