#ifndef _MESH_
#define _MESH_

#include "defines.h"
#include "Linear.h"
#include "BBox3f.h"

// Typedef used for color information
typedef uchar vec3uc[3];

struct Triangle {
   int vindex1, vindex2, vindex3;
   Vec3f norm;
};

struct Vertex {
   Vec3f coord;
   Vec3f norm;
   
   float confidence;

   vec3uc matDiff;
   
   // Topological information
   Triangle **tris;
   uchar numTris;
   uchar maxTris;
   
   Vertex **verts;
   float *edgeLengths;
   uchar numVerts;
   uchar maxVerts;
   
   // Ignore these...
   float distToBoundary;
   uchar count;
   uchar on_edge;
   uchar holeFill;
   signed char stepsToEdge;
};


class Mesh {

  public:

    int numVerts;
    Vertex *verts;

    int numTris;
    Triangle *tris;

    int hasConfidence;
    int hasNormals;
    int hasColor;
    int hasIntensity;

    Quaternion quat;
    Vec3f trans;

    BBox3f bbox;

    Mesh();
    ~Mesh();

    int readPly(char *filename);
    int writePly(char *filename);

    // Compute triangle and vertex normals.
    void initNormals();

    // Compute triangle normals.
    void computeTriNormals();

    // Compute vertex normals as average of triangle normals.
    void computeVertNormals();

    // Compute axis-aligned bounding box for whole mesh
    void computeBBox();

    
    // Apply a transformation to all the vertices and normals of a mesh
    void applyXform(Matrix4f &mat);
};

#endif

