#include "globals.h"
#include "lorensen_cases.h"

// Set up the edge table
void 
initEdgeTable()
{
   int i, ntris, signCase;
   int *edgeList, edge;
   
   // Consider each case of sign choices for 8 voxels in a cube
   for (signCase = 0; signCase < 256; signCase++) {
      
      // Parse Lorensen's edge table
      edgeList = polyCases[signCase].edges;
      ntris = 0;
      while (ntris < 5) {
	 if (edgeList[ntris*4] == 0 && edgeList[ntris*4+1] == 0 
	     && edgeList[ntris*4+2] == 0)
	    break;
	 
	 ntris++;
      }
      
      
      // Start filling the edge table
      TheEdgeTable[signCase].numTris = ntris;

      // Two cases (all negative and all positive) yield no triangles
      if (ntris == 0)
	 continue;
      
      // Init all edges to false
      for (i = 0; i < 12; i++)
	 TheEdgeTable[signCase].edgeBool[i] = FALSE;
      
      // Fill in the edge indices that comprise each triangle and
      // mark each edge that belongs to a triangle as TRUE
      for (i = 0; i < ntris; i++) {

	 // Need to subtract 1 because Lorensen table starts counting at 1
	 edge = *edgeList-1;
	 TheEdgeTable[signCase].tris[i].vindex1 = edge;
	 TheEdgeTable[signCase].edgeBool[edge] = TRUE;
	 edgeList++;
	 
	 edge = *edgeList-1;
	 TheEdgeTable[signCase].tris[i].vindex2 = edge;
	 TheEdgeTable[signCase].edgeBool[edge] = TRUE;
	 edgeList++;
	 
	 edge =*edgeList-1;
	 TheEdgeTable[signCase].tris[i].vindex3 = edge;
	 TheEdgeTable[signCase].edgeBool[edge] = TRUE;
	 edgeList++;
	 edgeList++;
      }
   }
}


void
printEdgeTable(FILE *fp)
{
   int i, j, ntris;

   for (i = 0; i < 256; i++) {
      ntris = TheEdgeTable[i].numTris;
      fprintf(fp, "Case=%d Ntris=%d", i, ntris);

      if (ntris == 0) {
	 fprintf(fp, "\n");
	 continue;
      }

      fprintf(fp, " Edges=(");
      for (j = 0; j < 12; j++) {
	 if (TheEdgeTable[i].edgeBool[j])
	    fprintf(fp, "%d ",j);
      }
      fprintf(fp, ") Tris=");

      for (j = 0; j < TheEdgeTable[i].numTris; j++) {
	 fprintf(fp, "(%d,%d,%d) ", TheEdgeTable[i].tris[j].vindex1,
		TheEdgeTable[i].tris[j].vindex2,
		TheEdgeTable[i].tris[j].vindex3);
      }
      fprintf(fp, "\n");
   }
}

